//
// Copyright (c) 2004-2010 StreamBase Systems, Inc. All rights reserved.
//
package com.streambase.contrib.feedsimplugin.regex;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.regex.PatternSyntaxException;
import java.util.zip.GZIPInputStream;

import com.streambase.sb.feedsim.FeedSimCSVInputStream;
import com.streambase.sb.util.Msg;

/**
 * This class will parse a file using the provided regular expression into comma separated values as returned by
 * the read() method. This particular implementation assumes the first line of the supplied file has an encoded date
 * which should be prepended to the time stamp field (the first field). This kind of representation is not uncommon
 * in historical market data files.
 * 
 * This class is intended to be extended by a very thin super class that basically just defines the regular
 * expression to use. 
 * 
 */
public class FeedSimRegExPlugin extends FeedSimCSVInputStream {

    private BufferedReader reader = null;
    private String currentLine=null;
    // Regex-related objects
    private Pattern _compiledFormat;
    
    private int pos=0;
    private int lineCount=1;
    
    private String date=null;
	
    /**
     * The constructor uses the file extension to determines if the data is compressed or not.
     * 
     * @param path - path to the data file
     * @param RegEx - The regular expression to be used to parse the lines
     * @throws IOException
     */
    public FeedSimRegExPlugin(String path, String RegEx) throws IOException {
    	super(path);

    	try {
    		_compiledFormat = Pattern.compile(RegEx);
    	} catch (PatternSyntaxException e) {
    		throw new IOException(e);
    	}
    	 
    	String extention = null;
    	if (path.length() > 4) {
    		extention=path.substring(path.length()-3, path.length());
    	}
    	
    	if ((extention != null) && ((".gz".endsWith(extention) || ".gzip".endsWith(extention)))) {
    		reader = new BufferedReader(new InputStreamReader(new GZIPInputStream(new FileInputStream(path))));
    	} else {
    		reader = new BufferedReader(new FileReader(path));
    	}
    	
    	// Read the date for this file. 
    	// N04012008
    	String rawDate=reader.readLine().trim();
    	date = rawDate.substring(5) + "-" + rawDate.substring(1, 3) + "-" + rawDate.substring(3,5) + " ";
    }
 
    public void close() throws IOException {
    	reader.close();
    	super.close();
    }
    
    // read will parse a new line if the previous one has been entirely delivered
    public int read() throws IOException {

        if (currentLine == null) {
        	while(true) {
        		String rawLine = reader.readLine();
        		lineCount++;
	        	if (rawLine == null) {
	        		return -1;
	        	}
	        	currentLine = parseLine(rawLine);
	        	if (currentLine != null) {
	        		break;
	        	} else {
	        		Msg.warn(Msg.format("Could not parse, skipping line {0}: {1}", lineCount, rawLine));
	        	}
        	}        	
        }
        
        if (pos >= currentLine.length()) {
            pos=0;
            currentLine=null;
        	return '\n';
        }
        return currentLine.charAt(pos++);
    }
    
    
    private String parseLine(String line) {
        Matcher match = _compiledFormat.matcher(line);

        if (!match.matches()) {
            return null;
        }

        StringBuilder ret = new StringBuilder();
        
        for (int i = 1; i < match.groupCount() ; i++) {
        	String s = match.group(i);
        	if (s == null)
        		return ret.toString();
        	if (i == 1) {
        		s = "'" + date + s + "'";
        	}
        	ret.append(s);
        	ret.append(",");
        }
        return ret.toString();
    }

 }